#include <iostream>
#include <vector>
using namespace std;

const int max_n = 600;

int n, m;

// DP state; shows whether a position is a winning one; positions are indexed by
// 1. number of single vertices (no edges)
// 2. number of pairs (two vertices connected to each other)
// 3. number of triples (paths of three verices)
// 4. whether there is a star (a vertex that connectes to three or more vertices not connected to any others)
bool dp[1 + max_n][1 + max_n / 2][1 + max_n / 3][2];

void fill_dp() {
	for (int singles = 0; singles <= n; ++singles) {
		for (int pairs = 0; pairs * 2 + singles <= n; ++pairs) {
			for (int triples = 0; triples * 3 + pairs * 2 + singles <= n; ++triples) {
				for (int star : {0, 1}) {
					// We go through all possible moves that do not immediately create a path of four vertices.
					// If we did, the opponent could immediately make a cycle out of it and win.
					// For each move, we check, whether making it gives us a losing position.
					// If we find one, it means that the current position is winning.

					if (singles >= 2 and not dp[singles - 2][pairs + 1][triples][star]) {
						// We can make a pair by connecting two single vertices.
						dp[singles][pairs][triples][star] = true;
					} else if (singles >= 1 and pairs >= 1 and not dp[singles - 1][pairs - 1][triples + 1][star]) {
						// We can make a triple by connecting a single to a pair.
						dp[singles][pairs][triples][star] = true;
					} else if (singles >= 1 and triples >= 1 and not dp[singles - 1][pairs][triples - 1][1]) {
						// We can make a star by connecting a single to a triple.
						dp[singles][pairs][triples][star] = true;
					} else if (triples >= 1 and not dp[singles][pairs][triples - 1][star]) {
						// We can make a triangle by connecting the two unconnected vertices of a triple.
						// We can't add anything to a triangle without creating a path of four verices.
						// So we can just ignore it.
						dp[singles][pairs][triples][star] = true;
					} else if (singles >= 1 and star == 1 and not dp[singles - 1][pairs][triples][star]) {
						// If a star exists, we can connect a single to it and it remains a star.
						dp[singles][pairs][triples][star] = true;
					}
				}
			}
		}
	}
}

int main() {
	cin >> n >> m;

	vector<vector<int>> edges(1 + n);
	for (int i = 0; i < m; ++i) {
		int a, b;
		cin >> a >> b;
		edges[a].push_back(b);
		edges[b].push_back(a);
	}

	// Can we win on first move? If so, output the answer and exit.
	for (int i1 = 1; i1 <= n; ++i1) {
		for (int i2 : edges[i1]) {
			for (int i3 : edges[i2]) {
				if (i3 == i1) continue;
				for (int i4 : edges[i3]) {
					if (i4 == i1 or i4 == i2) continue;
					cout << i1 << ' ' << i4 << '\n';
					return 0;
				}
			}
		}
	}

	fill_dp();

	// We have already checked that it is not possible to win in one move.
	// Now we calculate how many of the graph components of interest to us exist.
	// Since there is no path of four vertices, it is easy to prove that only the
	// singles, pairs, triples, stars and triangles are possible.
	int singles = 0, pairs = 0, triples = 0, stars = 0;

	// We store the number of one vertex of each type, so that later we know which
	// vertex we can connect to which, except we store two singles.
	int single_1 = -1, single_2 = -1, pair_member = -1, triple_center = -1, star_center = -1;

	for (int i = 1; i <= n; ++i) {
		if (edges[i].size() == 0) {
			++singles;
			single_2 = single_1;
			single_1 = i;
			continue;
		}
		if (edges[i].size() == 1) {
			int neighbor = edges[i][0];
			if (edges[neighbor].size() == 1) {
				// This is a pair. To avoid double counting, we only count the pair for one member.
				if (i < neighbor) {
					++pairs;
					pair_member = i;
				}
			}
			// It the neighbor has more edges, we check that component when we examine the neigbor.
			continue;
		}
		// Now we know that this vertex has at least two edges.
		// So it either belongs to a triangle or is the center of a triple or a star.
		if (edges[edges[i][0]].size() > 1) {
			// If a neighbor has more than one edge, this component can only be a triangle.
			// We are not interested in triangles because we cannot add anything to them
			// without creating a winning position for the opponent.
			continue;
		}
		if (edges[i].size() == 2) {
			++triples;
			triple_center = i;
		} else {
			++stars;
			star_center = i;
		}
	}

	int star = min(stars, 1);
	if (not dp[singles][pairs][triples][star]) {
		cout << 0 << '\n';
		return 0;
	}

	// The same checks as in the fill_dp() function.
	if (singles >= 2 and not dp[singles - 2][pairs + 1][triples][star]) {
		cout << single_1 << ' ' << single_2 << '\n';
	} else if (singles >= 1 and pairs >= 1 and not dp[singles - 1][pairs - 1][triples + 1][star]) {
		cout << single_1 << ' ' << pair_member << '\n';
	} else if (singles >= 1 and triples >= 1 and not dp[singles - 1][pairs][triples - 1][1]) {
		cout << single_1 << ' ' << triple_center << '\n';
	} else if (triples >= 1 and not dp[singles][pairs][triples - 1][star]) {
		cout << edges[triple_center][0] << ' ' << edges[triple_center][1] << '\n';
	} else if (singles >= 1 and star == 1 and not dp[singles - 1][pairs][triples][star]) {
		cout << single_1 << ' ' << star_center << '\n';
	}
}
