# Straightforward copy of the C++ solution.
# However, the overhead of Python lists compared to C++ arrays is such
# that we can't afford to set the time and memory limits to let this
# solution pass and it will score about 40%.

n, m = (int(x) for x in input().split())

edges = [[] for _ in range(1 + n)]
for i in range(m):
    a, b = (int(x) for x in input().split())
    edges[a].append(b)
    edges[b].append(a)

# Can we win on first move? If so, output the answer and exit.
for i1 in range(1, 1 + n):
    for i2 in edges[i1]:
        for i3 in edges[i2]:
            if i3 == i1: continue
            for i4 in edges[i3]:
                if i4 == i1 or i4 == i2: continue
                print(i1, i4)
                exit(0)

# DP state; shows whether a position is a winning one; positions are indexed by
# 1. number of single vertices (no edges)
# 2. number of pairs (two vertices connected to each other)
# 3. number of triples (paths of three verices)
# 4. whether there is a star (a vertex that connectes to three or more vertices not connected to any others)

# Note that changing the order of indices from dp[singles][pairs][triples][star]
# to dp[star][triples][pairs][singles] would reduce the whole data structure from
# ~600*300*200=36e6 lists (each having 2 elements, for a total of 72e6 Booleans)
# to ~2*200*300=12e4 lists (now each having 600 elements, so still 72e6 Booleans
# in total) for ~4-fold decrease in the overall memory footprint of the solution;
# but this would still not achieve any additional subtasks.

dp = [[[[False
        for _ in range(2)]
        for _ in range(1 + n // 3)]
        for _ in range(1 + n // 2)]
        for _ in range(1 + n)]

for singles in range(1 + n):
    for pairs in range(1 + (n - singles) // 2):
        for triples in range(1 + (n - singles - 2 * pairs) // 3):
            for star in [0, 1]:
                # We go through all possible moves that do not immediately create a path of four vertices.
                # If we did, the opponent could immediately make a cycle out of it and win.
                # For each move, we check, whether making it gives us a losing position.
                # If we find one, it means that the current position is winning.

                if singles >= 2 and not dp[singles - 2][pairs + 1][triples][star]:
                    # We can make a pair by connecting two single vertices.
                    dp[singles][pairs][triples][star] = True
                elif singles >= 1 and pairs >= 1 and not dp[singles - 1][pairs - 1][triples + 1][star]:
                    # We can make a triple by connecting a single to a pair.
                    dp[singles][pairs][triples][star] = True
                elif singles >= 1 and triples >= 1 and not dp[singles - 1][pairs][triples - 1][1]:
                    # We can make a star by connecting a single to a triple.
                    dp[singles][pairs][triples][star] = True
                elif triples >= 1 and not dp[singles][pairs][triples - 1][star]:
                    # We can make a triangle by connecting the two unconnected vertices of a triple.
                    # We can't add anything to a triangle without creating a path of four verices.
                    # So we can just ignore it.
                    dp[singles][pairs][triples][star] = True
                elif singles >= 1 and star == 1 and not dp[singles - 1][pairs][triples][star]:
                    # If a star exists, we can connect a single to it and it remains a star.
                    dp[singles][pairs][triples][star] = True

# We have already checked that it is not possible to win in one move.
# Now we calculate how many of the graph components of interest to us exist.
# Since there is no path of four vertices, it is easy to prove that only the
# singles, pairs, triples, stars and triangles are possible.
singles, pairs, triples, stars = 0, 0, 0, 0

# We store the number of one vertex of each type, so that later we know which
# vertex we can connect to which, except we store two singles.
single_1, single_2, pair_member, triple_center, star_center = -1, -1, -1, -1, -1

for i in range(1, 1 + n):
    if len(edges[i]) == 0:
        singles += 1
        single_2, single_1 = single_1, i
        continue
    if len(edges[i]) == 1:
        neighbor = edges[i][0]
        if len(edges[neighbor]) == 1:
            # This is a pair. To avoid double counting, we only count the pair for one member.
            if i < neighbor:
                pairs += 1
                pair_member = i
        # It the neighbor has more edges, we check that component when we examine the neigbor.
        continue
    # Now we know that this vertex has at least two edges.
    # So it either belongs to a triangle or is the center of a triple or a star.
    if len(edges[edges[i][0]]) > 1:
        # If a neighbor has more than one edge, this component can only be a triangle.
        # We are not interested in triangles because we cannot add anything to them
        # without creating a winning position for the opponent.
        continue
    if len(edges[i]) == 2:
        triples += 1
        triple_center = i
    else:
        stars += 1
        star_center = i

star = min(stars, 1)
if not dp[singles][pairs][triples][star]:
    print(0)
    exit(0)

# The same checks as in the fill_dp() function.
if singles >= 2 and not dp[singles - 2][pairs + 1][triples][star]:
    print(single_1, single_2)
elif singles >= 1 and pairs >= 1 and not dp[singles - 1][pairs - 1][triples + 1][star]:
    print(single_1, pair_member)
elif singles >= 1 and triples >= 1 and not dp[singles - 1][pairs][triples - 1][1]:
    print(single_1, triple_center)
elif triples >= 1 and not dp[singles][pairs][triples - 1][star]:
    print(edges[triple_center][0], edges[triple_center][1])
elif singles >= 1 and star == 1 and not dp[singles - 1][pairs][triples][star]:
    print(single_1, star_center)
